@@ -705,6 +705,13 @@ struct workqueue_struct *snd_compat_create_workqueue2(const char *name);
 #define create_workqueue(name) snd_compat_create_workqueue2(name)
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)
+#include <linux/workqueue.h>
+#ifndef create_singlethread_workqueue
+#define create_singlethread_workqueue(name) create_workqueue(name)
+#endif
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20)
 #include <linux/workqueue.h>
 /* redefine INIT_WORK() */
@@ -884,7 +891,7 @@ static inline unsigned long msecs_to_jiffies(const unsigned int m)
 #define snd_dma_continuous_data(x)	((struct device *)(unsigned long)(x))
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 17)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 24)
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)
 #include <linux/mm.h>
 #include <asm/io.h>
@@ -907,6 +914,9 @@ static inline unsigned long msecs_to_jiffies(const unsigned int m)
 #ifndef DMA_24BIT_MASK
 #define DMA_24BIT_MASK	0x0000000000ffffffULL
 #endif
+#ifndef DMA_BIT_MASK
+#define DMA_BIT_MASK(n)	(((n) == 64) ? ~0ULL : ((1ULL<<(n))-1))
+#endif
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
@@ -1596,6 +1606,9 @@ static inline u64 get_unaligned_be64(void *p)
 #ifndef upper_32_bits
 #define upper_32_bits(n) ((u32)(((n) >> 16) >> 16))
 #endif
+#ifndef lower_32_bits
+#define lower_32_bits(n) ((u32)(n))
+#endif
 
 #ifndef CONFIG_HAVE_PAGE_TO_PFN
 #define page_to_pfn(page)       (page_to_phys(page) >> PAGE_SHIFT)
@@ -1760,10 +1773,30 @@ static inline void *pci_ioremap_bar(struct pci_dev *pdev, int bar)
 /*
  * definition of type 'bool'
  */
+#ifndef CONFIG_SND_HAS_BUILTIN_BOOL
+typedef int _Bool;
+#endif
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
 #ifndef bool	/* just to be sure */
 typedef _Bool bool;
 #endif
 #endif
 
+/* memdup_user() wrapper */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 30)
+#include <linux/err.h>
+#include <asm/uaccess.h>
+static inline void *memdup_user(void __user *src, size_t len)
+{
+	void *p = kmalloc(len, GFP_KERNEL);
+	if (!p)
+		return ERR_PTR(-ENOMEM);
+	if (copy_from_user(p, src, len)) {
+		kfree(p);
+		return ERR_PTR(-EFAULT);
+	}
+	return p;
+}
+#endif
+
 #endif /* __SOUND_LOCAL_DRIVER_H */
