@@ -1,6 +1,6 @@
 #!/bin/bash
 
-SCRIPT_VERSION=0.4.53
+SCRIPT_VERSION=0.4.56
 CHANGELOG="http://www.alsa-project.org/alsa-info.sh.changelog"
 
 #################################################################################
@@ -44,54 +44,67 @@ pbcheck(){
 }
 
 update() {
-	wget -O /tmp/alsa-info.sh "http://www.alsa-project.org/alsa-info.sh" >/dev/null 2>&1
-	REMOTE_VERSION=`grep SCRIPT_VERSION /tmp/alsa-info.sh |head -n1 |sed 's/.*=//'`
+	SHFILE=`mktemp -p /tmp alsa-info.XXXXXXXXXX`
+	wget -O $SHFILE "http://www.alsa-project.org/alsa-info.sh" >/dev/null 2>&1
+	REMOTE_VERSION=`grep SCRIPT_VERSION $SHFILE |head -n1 |sed 's/.*=//'`
 	if [ "$REMOTE_VERSION" != "$SCRIPT_VERSION" ]; then
 		if [[ -n $DIALOG ]]
 		then
+			OVERWRITE=
 			if [ -w $0 ]; then
-				dialog --yesno "Newer version of ALSA-Info has been found\n\nDo you wish to install it?" 0 0
-			else
+				dialog --yesno "Newer version of ALSA-Info has been found\n\nDo you wish to install it?\nNOTICE: The original file $0 will be overwritten!" 0 0
+				DIALOG_EXIT_CODE=$?
+				if [[ $DIALOG_EXIT_CODE = 0 ]]; then
+				  OVERWRITE=yes
+				fi
+			fi
+			if [ -z "$OVERWRITE" ]; then
 				dialog --yesno "Newer version of ALSA-Info has been found\n\nDo you wish to download it?" 0 0
+				DIALOG_EXIT_CODE=$?
 			fi
-			DIALOG_EXIT_CODE=$?
 			if [[ $DIALOG_EXIT_CODE = 0 ]]
 			then
 				echo "Newer version detected: $REMOTE_VERSION"
 				echo "To view the ChangeLog, please visit $CHANGELOG"
-				if [ -w $0 ]; then
-					cp /tmp/alsa-info.sh $0
+				if [ "$OVERWRITE" = "yes" ]; then
+					cp $SHFILE $0
 					echo "ALSA-Info script has been updated to v $REMOTE_VERSION"
 					echo "Please re-run the script"
-					rm /tmp/alsa-info.sh 2>/dev/null
+					rm $SHFILE 2>/dev/null
 				else
+					mv $SHFILE /tmp/alsa-info.sh || exit 1
 					echo "ALSA-Info script has been downloaded as /tmp/alsa-info.sh."
 					echo "Please re-run the script from new location."
 				fi
 				exit
 			else
-				rm /tmp/alsa-info.sh 2>/dev/null
+				rm $SHFILE 2>/dev/null
 			fi
 		else
 			echo "Newer version detected: $REMOTE_VERSION"
 			echo "To view the ChangeLog, please visit $CHANGELOG"
 			if [ -w $0 ]; then
-				cp /tmp/alsa-info.sh $0
+				echo "The original file $0 will be overwritten!"
+				echo -n "If you do not like to proceed, press Ctrl-C now.." ; read inp
+				cp $SHFILE $0
 				echo "ALSA-Info script has been updated. Please re-run it."
-				rm /tmp/alsa-info.sh 2>/dev/null
+				rm $SHFILE 2>/dev/null
 			else
+				mv $SHFILE /tmp/alsa-info.sh || exit 1
 				echo "ALSA-Info script has been downloaded as /tmp/alsa-info.sh."
 				echo "Please, re-run it from new location."
 			fi
 			exit
 		fi
 	else
-		rm /tmp/alsa-info.sh 2>/dev/null
+		rm $SHFILE 2>/dev/null
 	fi
 }
 
 cleanup() {
-	rm -r $TEMPDIR 2>/dev/null
+	if [ -n "$TEMPDIR" -a "$KEEP_FILES" != "yes" ]; then
+		rm -r "$TEMPDIR" 2>/dev/null
+	fi
 }
 
 
@@ -123,7 +136,7 @@ withamixer() {
         echo "!!-------------" >> $FILE
         echo "" >> $FILE
 	for i in `grep "]: " /proc/asound/cards | awk -F ' ' '{ print $1} '` ; do
-	CARD_NAME=`grep "^ *$i " /tmp/alsainfo/alsacards.tmp|awk {'print $2'}`
+	CARD_NAME=`grep "^ *$i " $TEMPDIR/alsacards.tmp|awk {'print $2'}`
 	echo "!!-------Mixer controls for card $i $CARD_NAME]" >> $FILE
 	echo "" >>$FILE
 	amixer -c$i info>> $FILE 2>&1
@@ -147,9 +160,9 @@ withalsactl() {
         if [ -z "$exe" ]; then
         	exe=`whereis alsactl | cut -d ' ' -f 2`
         fi
-	$exe -f /tmp/alsainfo/alsactl.tmp store
+	$exe -f $TEMPDIR/alsactl.tmp store
 	echo "--startcollapse--" >> $FILE
-	cat /tmp/alsainfo/alsactl.tmp >> $FILE
+	cat $TEMPDIR/alsactl.tmp >> $FILE
 	echo "--endcollapse--" >> $FILE
 	echo "" >> $FILE
 	echo "" >> $FILE
@@ -201,6 +214,67 @@ then
 fi
 }
 
+withsysfs() {
+    local i f
+    local printed=""
+    for i in /sys/class/sound/*; do
+	case "$i" in
+	    */hwC?D?)
+		if [ -f $i/init_pin_configs ]; then
+		    if [ -z "$printed" ]; then
+			echo "!!Sysfs Files" >> $FILE
+			echo "!!-----------" >> $FILE
+			echo "" >> $FILE
+		    fi
+		    for f in init_pin_configs driver_pin_configs user_pin_configs init_verbs; do
+			echo "$i/$f:" >> $FILE
+			cat $i/$f >> $FILE
+			echo >> $FILE
+		    done
+		    printed=yes
+		fi
+		;;
+	    esac
+    done
+    if [ -n "$printed" ]; then
+	echo "" >> $FILE
+    fi
+}
+
+get_alsa_library_version() {
+	ALSA_LIB_VERSION=`grep VERSION_STR /usr/include/alsa/version.h 2>/dev/null|awk {'print $3'}|sed 's/"//g'`
+
+	if [ -z "$ALSA_LIB_VERSION" ]; then
+		if [ -f /etc/lsb-release ]; then
+			. /etc/lsb-release
+			case "$DISTRIB_ID" in
+				Ubuntu)
+					if which dpkg > /dev/null ; then
+						ALSA_LIB_VERSION=`dpkg -l libasound2 | tail -1 | awk '{print $3}' | cut -f 1 -d -`
+					fi
+
+					if [ "$ALSA_LIB_VERSION" = "<none>" ]; then
+						ALSA_LIB_VERSION=""
+					fi
+					return
+					;;
+				*)
+					return
+					;;
+			esac
+		elif [ -f /etc/debian_version ]; then
+			if which dpkg > /dev/null ; then
+				ALSA_LIB_VERSION=`dpkg -l libasound2 | tail -1 | awk '{print $3}' | cut -f 1 -d -`
+			fi
+
+			if [ "$ALSA_LIB_VERSION" = "<none>" ]; then
+				ALSA_LIB_VERSION=""
+			fi
+			return
+		fi
+	fi
+}
+
 
 #Run checks to make sure the programs we need are installed.
 LSPCI=$(which lspci 2>/dev/null| sed 's|^[^/]*||' 2>/dev/null);
@@ -237,6 +311,12 @@ case "$1" in
 		REPEAT=""
 		shift
 		;;
+	--stdout)
+		DIALOG=""
+		NOUPLOAD="yes"
+		QUESTION="no"
+		TOSTDOUT="yes"
+		;;
 esac
 done
 		
@@ -285,15 +365,18 @@ fi
 fi # question
 
 #Set the output file
-TEMPDIR="/tmp/alsainfo/"
-FILE="/tmp/alsa-info.txt"
+TEMPDIR=`mktemp -p /tmp -d alsa-info.XXXXXXXXXX`
+FILE="$TEMPDIR/alsa-info.txt"
+NFILE="/tmp/alsa-info.txt"
+
+trap cleanup 0
 
 if [ "$PROCEED" = "yes" ]; then
 
 if [[ -z "$LSPCI" ]] 
-	then
+then
 	echo "This script requires lspci. Please install it, and re-run this script."
-exit 0
+	exit 0
 fi
 
 #Create the temporary work dir.
@@ -307,26 +390,27 @@ KERNEL_MACHINE=`uname -m`
 KERNEL_OS=`uname -o`
 [[ `uname -v |grep SMP`  ]] && KERNEL_SMP="Yes" || KERNEL_SMP="No" 
 ALSA_DRIVER_VERSION=`cat /proc/asound/version |head -n1|awk {'print $7'} |sed 's/\.$//'`
-ALSA_LIB_VERSION=`grep VERSION_STR /usr/include/alsa/version.h 2>/dev/null|awk {'print $3'}|sed 's/"//g'`
+get_alsa_library_version
 ALSA_UTILS_VERSION=`amixer -v |awk {'print $3'}`
-VENDOR_ID=`lspci -vn |grep 040[1-3] | awk -F':' '{print $3}'|awk {'print substr($0, 2);}' >/tmp/alsainfo/vendor_id.tmp`
-DEVICE_ID=`lspci -vn |grep 040[1-3] | awk -F':' '{print $4}'|awk {'print $1'} >/tmp/alsainfo/device_id.tmp`
+VENDOR_ID=`lspci -vn |grep 040[1-3] | awk -F':' '{print $3}'|awk {'print substr($0, 2);}' >$TEMPDIR/vendor_id.tmp`
+DEVICE_ID=`lspci -vn |grep 040[1-3] | awk -F':' '{print $4}'|awk {'print $1'} >$TEMPDIR/device_id.tmp`
 LAST_CARD=$((`grep "]: " /proc/asound/cards | wc -l` - 1 ))
 
 ESDINST=$(which esd 2>/dev/null| sed 's|^[^/]*||' 2>/dev/null)
 PAINST=$(which pulseaudio 2>/dev/null| sed 's|^[^/]*||' 2>/dev/null)
 ARTSINST=$(which artsd 2>/dev/null| sed 's|^[^/]*||' 2>/dev/null)
+JACKINST=$(which jackd 2>/dev/null| sed 's|^[^/]*||' 2>/dev/null)
 
-cat /proc/asound/modules 2>/dev/null|awk {'print $2'}>/tmp/alsainfo/alsamodules.tmp
-cat /proc/asound/cards >/tmp/alsainfo/alsacards.tmp
-lspci |grep -i "multi\|audio">/tmp/alsainfo/lspci.tmp
+cat /proc/asound/modules 2>/dev/null|awk {'print $2'}>$TEMPDIR/alsamodules.tmp
+cat /proc/asound/cards >$TEMPDIR/alsacards.tmp
+lspci |grep -i "multi\|audio">$TEMPDIR/lspci.tmp
 
 #Check for HDA-Intel cards codec#*
-cat /proc/asound/card*/codec\#* > /tmp/alsainfo/alsa-hda-intel.tmp 2> /dev/null
+cat /proc/asound/card*/codec\#* > $TEMPDIR/alsa-hda-intel.tmp 2> /dev/null
 
 #Check for AC97 cards codec
-cat /proc/asound/card*/codec97\#0/ac97\#0-0 > /tmp/alsainfo/alsa-ac97.tmp 2> /dev/null
-cat /proc/asound/card*/codec97\#0/ac97\#0-0+regs > /tmp/alsainfo/alsa-ac97-regs.tmp 2> /dev/null
+cat /proc/asound/card*/codec97\#0/ac97\#0-0 > $TEMPDIR/alsa-ac97.tmp 2> /dev/null
+cat /proc/asound/card*/codec97\#0/ac97\#0-0+regs > $TEMPDIR/alsa-ac97-regs.tmp 2> /dev/null
 
 #Fetch the info, and put it in $FILE in a nice readable format.
 if [[ -z $PASTEBIN ]]; then
@@ -368,34 +452,41 @@ echo "" >> $FILE
 echo "!!Loaded ALSA modules" >> $FILE
 echo "!!-------------------" >> $FILE
 echo "" >> $FILE
-cat /tmp/alsainfo/alsamodules.tmp >> $FILE
+cat $TEMPDIR/alsamodules.tmp >> $FILE
 echo "" >> $FILE
 echo "" >> $FILE
 echo "!!Sound Servers on this system" >> $FILE
 echo "!!----------------------------" >> $FILE
 echo "" >> $FILE
 if [[ -n $PAINST ]];then
-[[ `pgrep $PAINST` ]] && PARUNNING="Yes" || PARUNNING="No"
+[[ `pgrep '^(.*/)?pulseaudio$'` ]] && PARUNNING="Yes" || PARUNNING="No"
 echo "Pulseaudio:" >> $FILE
 echo "      Installed - Yes ($PAINST)" >> $FILE
 echo "      Running - $PARUNNING" >> $FILE
 echo "" >> $FILE
 fi
 if [[ -n $ESDINST ]];then
-[[ `pgrep $ESDINST` ]] && ESDRUNNING="Yes" || ESDRUNNING="No"
+[[ `pgrep '^(.*/)?esd$'` ]] && ESDRUNNING="Yes" || ESDRUNNING="No"
 echo "ESound Daemon:" >> $FILE
 echo "      Installed - Yes ($ESDINST)" >> $FILE
 echo "      Running - $ESDRUNNING" >> $FILE
 echo "" >> $FILE
 fi
 if [[ -n $ARTSINST ]];then
-[[ `pgrep $ARTSINST` ]] && ARTSRUNNING="Yes" || ARTSRUNNING="No"
+[[ `pgrep '^(.*/)?artsd$'` ]] && ARTSRUNNING="Yes" || ARTSRUNNING="No"
 echo "aRts:" >> $FILE
 echo "      Installed - Yes ($ARTSINST)" >> $FILE
 echo "      Running - $ARTSRUNNING" >> $FILE
 echo "" >> $FILE
 fi
-if [[ -z "$PAINST" && -z "$ESDINST" && -z "$ARTSINST" ]];then
+if [[ -n $JACKINST ]];then
+[[ `pgrep '^(.*/)?jackd$'` ]] && JACKRUNNING="Yes" || JACKRUNNING="No"
+echo "Jack:" >> $FILE
+echo "      Installed - Yes ($JACKINST)" >> $FILE
+echo "      Running - $JACKRUNNING" >> $FILE
+echo "" >> $FILE
+fi
+if [[ -z "$PAINST" && -z "$ESDINST" && -z "$ARTSINST" && -z "$JACKINST" ]];then
 echo "No sound servers found." >> $FILE
 echo "" >> $FILE
 fi
@@ -403,13 +494,13 @@ echo "" >> $FILE
 echo "!!Soundcards recognised by ALSA" >> $FILE
 echo "!!-----------------------------" >> $FILE
 echo "" >> $FILE
-cat /tmp/alsainfo/alsacards.tmp >> $FILE
+cat $TEMPDIR/alsacards.tmp >> $FILE
 echo "" >> $FILE
 echo "" >> $FILE
 echo "!!PCI Soundcards installed in the system" >> $FILE
 echo "!!--------------------------------------" >> $FILE
 echo "" >> $FILE
-cat /tmp/alsainfo/lspci.tmp >> $FILE
+cat $TEMPDIR/lspci.tmp >> $FILE
 echo "" >> $FILE
 echo "" >> $FILE
 echo "!!Advanced information - PCI Vendor/Device/Susbsystem ID's" >> $FILE
@@ -442,27 +533,27 @@ done
 echo "" >> $FILE
 fi
 
-if [ -s "/tmp/alsainfo/alsa-hda-intel.tmp" ] 
+if [ -s "$TEMPDIR/alsa-hda-intel.tmp" ] 
 then
 	echo "!!HDA-Intel Codec information" >> $FILE
 	echo "!!---------------------------" >> $FILE
 	echo "--startcollapse--" >> $FILE
 	echo "" >> $FILE
-	cat /tmp/alsainfo/alsa-hda-intel.tmp >> $FILE
+	cat $TEMPDIR/alsa-hda-intel.tmp >> $FILE
 	echo "--endcollapse--" >> $FILE
 	echo "" >> $FILE
 	echo "" >> $FILE
 fi
 
-if [ -s "/tmp/alsainfo/alsa-ac97.tmp" ]
+if [ -s "$TEMPDIR/alsa-ac97.tmp" ]
 then
         echo "!!AC97 Codec information" >> $FILE
         echo "!!---------------------------" >> $FILE
         echo "--startcollapse--" >> $FILE
         echo "" >> $FILE
-        cat /tmp/alsainfo/alsa-ac97.tmp >> $FILE
+        cat $TEMPDIR/alsa-ac97.tmp >> $FILE
         echo "" >> $FILE
-        cat /tmp/alsainfo/alsa-ac97-regs.tmp >> $FILE
+        cat $TEMPDIR/alsa-ac97-regs.tmp >> $FILE
         echo "--endcollapse--" >> $FILE
 	echo "" >> $FILE
 	echo "" >> $FILE
@@ -498,6 +589,7 @@ then
         		withamixer
         		withalsactl
         		withlsmod
+			withsysfs
         		pbcheck
 			;;
 		--update)
@@ -506,13 +598,13 @@ then
 			;;
 		--no-upload)
 			NOUPLOAD="yes"
-			KEEP_FILES="yes"
 			withdevices
 			withconfigs
 			withaplay
 			withamixer
 			withalsactl
 			withlsmod
+			withsysfs
 			;;
 		--debug)
 			echo "Debugging enabled. $FILE and $TEMPDIR will not be deleted"
@@ -524,6 +616,7 @@ then
 			withamixer
 			withalsactl
 			withlsmod
+			withsysfs
 			;;
 		--with-all)
 			withdevices
@@ -532,6 +625,7 @@ then
 			withamixer
 			withalsactl
 			withlsmod
+			withsysfs
 			;;
 		--with-aplay)
 			withaplay
@@ -573,6 +667,17 @@ then
 				fi
 			fi
 			;;
+		--stdout)
+			NOUPLOAD="yes"
+			withdevices
+			withconfigs
+			withaplay
+			withamixer
+			withalsactl
+			withlsmod
+			cat $FILE
+			rm $FILE
+			;;
 		--about)
 			echo "Written/Tested by the following users of #alsa on irc.freenode.net:"
 			echo ""
@@ -581,7 +686,7 @@ then
 			echo "	gnubien - Various script ideas / Testing"
 			echo "	GrueMaster - HDA Intel specific items / Testing"
 			echo "	olegfink - Script update function"
-			cleanup
+			echo "  TheMuso - display to stdout functionality"
 			exit 0
 			;;
 		*)
@@ -599,10 +704,11 @@ then
 			echo "	--no-upload (do not upload contents to remote server)"
 			echo "	--pastebin (use http://pastebin.ca) as remote server"
 			echo "	    instead www.alsa-project.org"
+			echo "  --stdout (print alsa information to standard output"
+			echo "      instead of a file)"
 			echo "	--about (show some information about the script)"
 			echo "	--debug (will run the script as normal, but will not"
 			echo "	     delete $FILE)"
-			cleanup
 			exit 0
 			;;
 	esac
@@ -610,127 +716,127 @@ then
 	done
 fi
 
-if [ "$PROCEED" = "yes" ]; then
+if [ "$PROCEED" = "no" ]; then
+	exit 1
+fi
+
+if [ -n "$NOUPLOAD" ]; then
+
+	if [ -z "$TOSTDOUT" ]; then
+		mv $FILE $NFILE || exit 1
+	fi
+
+	if [[ -n $DIALOG ]]
+	then
+		if [[ -n $PBERROR ]]; then
+			dialog --backtitle "$BGTITLE" --title "Information collected" --msgbox "An error occurred while contacting the $WWWSERVICE. Your information was NOT automatically uploaded.\n\nYour ALSA information can be seen by looking in $NFILE" 10 100
+		else
+			dialog --backtitle "$BGTITLE" --title "Information collected" --msgbox "You requested that your information was NOT automatically uploaded to the $WWWSERVICE\n\nYour ALSA information can be seen by looking in $NFILE" 10 100
+		fi
+	else
+		echo
+
+		if [[ -n $PBERROR ]]; then
+			echo "An error occurred while contacting the $WWWSERVICE. Your information was NOT automatically uploaded."
+			echo ""
+			echo "Your ALSA information can be seen by looking in $NFILE"
+			echo ""
+		else
+			if [ -z "$TOSTDOUT" ]; then
+				echo "You requested that your information was NOT automatically uploaded to the $WWWSERVICE"
+				echo ""
+				echo "Your ALSA information can be seen by looking in $NFILE"
+				echo ""
+			fi
+		fi
+	fi
+
+	exit
+
+fi # noupload
 
 #Test that wget is installed, and supports --post-file. Upload $FILE if it does, and prompt user to upload file if it doesnt. 
 if
 WGET=$(which wget 2>/dev/null| sed 's|^[^/]*||' 2>/dev/null); [[ -n "${WGET}" ]] && [[ -x "${WGET}" ]] && [[ `wget --help |grep post-file` ]]
 then
-if [[ -n "$DIALOG" ]]
+
+if [[ -n $DIALOG ]]
 then
-	if [[ -z $NOUPLOAD ]]; then
-	if [[ -z $PASTEBIN ]]; then
-	wget -O - --tries=5 --timeout=60 --post-file=/tmp/alsa-info.txt "http://www.alsa-project.org/cardinfo-db/" &>/tmp/alsainfo/wget.tmp || echo "Upload failed; exit"
+
+if [[ -z $PASTEBIN ]]; then
+	wget -O - --tries=5 --timeout=60 --post-file=$FILE "http://www.alsa-project.org/cardinfo-db/" &>$TEMPDIR/wget.tmp || echo "Upload failed; exit"
 	{ for i in 10 20 30 40 50 60 70 80 90; do
 		echo $i
 		sleep 0.2
 	done
 	echo; } |dialog --backtitle "$BGTITLE" --guage "Uploading information to www.alsa-project.org ..." 6 70 0
-	else
-	wget -O - --tries=5 --timeout=60 --post-file=/tmp/alsa-info.txt "http://pastebin.ca/quiet-paste.php?api=$PASTEBINKEY&encrypt=t&encryptpw=blahblah" &>/tmp/alsainfo/wget.tmp || echo "Upload failed; exit"
+else
+	wget -O - --tries=5 --timeout=60 --post-file=$FILE "http://pastebin.ca/quiet-paste.php?api=$PASTEBINKEY&encrypt=t&encryptpw=blahblah" &>$TEMPDIR/wget.tmp || echo "Upload failed; exit"
 	{ for i in 10 20 30 40 50 60 70 80 90; do
 		echo $i
 		sleep 0.2
 	done
 	echo; } |dialog --backtitle "$BGTITLE" --guage "Uploading information to www.pastebin.ca ..." 6 70 0
-	fi
-	fi
+fi
+
+dialog --backtitle "$BGTITLE" --title "Information uploaded" --yesno "Would you like to see the uploaded information?" 5 100 
+DIALOG_EXIT_CODE=$?
+if [ $DIALOG_EXIT_CODE = 0 ]; then
+	grep -v "alsa-info.txt" $FILE >$TEMPDIR/uploaded.txt
+	dialog --backtitle "$BGTITLE" --textbox $TEMPDIR/uploaded.txt 0 0
+fi
+
+clear
+
+# no dialog
 else
 
-	if [[ -z $NOUPLOAD ]]; then
-	if [[ -z $PASTEBIN ]]; then
+if [[ -z $PASTEBIN ]]; then
 	echo -n "Uploading information to www.alsa-project.org ... " 
-	wget -O - --tries=5 --timeout=60 --post-file=/tmp/alsa-info.txt http://www.alsa-project.org/cardinfo-db/ &>/tmp/alsainfo/wget.tmp &
-	else
+	wget -O - --tries=5 --timeout=60 --post-file=$FILE http://www.alsa-project.org/cardinfo-db/ &>$TEMPDIR/wget.tmp &
+else
 	echo -n "Uploading information to www.pastebin.ca ... " 
-	wget -O - --tries=5 --timeout=60 --post-file=/tmp/alsa-info.txt http://pastebin.ca/quiet-paste.php?api=$PASTEBINKEY &>/tmp/alsainfo/wget.tmp &
-	fi
-	fi
-fi
-#Progess spinner for wget transfer.
-if [[ -z "$DIALOG" ]]	
-then
-	i=1
-	sp="/-\|"
-	echo -n ' '
-	while pgrep wget &>/dev/null
-	do
-	echo -en "\b${sp:i++%${#sp}:1}"
-	done
+	wget -O - --tries=5 --timeout=60 --post-file=$FILE http://pastebin.ca/quiet-paste.php?api=$PASTEBINKEY &>$TEMPDIR/wget.tmp &
 fi
 
+#Progess spinner for wget transfer.
+i=1
+sp="/-\|"
+echo -n ' '
+while pgrep wget &>/dev/null
+do
+	echo -en "\b${sp:i++%${#sp}:1}"
+done
+
+echo -e "\b Done!"
+echo ""
+
+fi #dialog
+
 #See if tput is available, and use it if it is.	
-if [[ -z $NOUPLOAD ]]; then
-	if [[ -n "$TPUT" ]]
-	then
-		if [[ -z $PASTEBIN ]]; then
-			FINAL_URL=`tput setaf 1; grep "SUCCESS:" /tmp/alsainfo/wget.tmp | cut -d ' ' -f 2 ; tput sgr0`
-		else
-			FINAL_URL=`tput setaf 1; grep "SUCCESS:" /tmp/alsainfo/wget.tmp |sed -n 's/.*\:\([0-9]\+\).*/http:\/\/pastebin.ca\/\1/p';tput sgr0`
-		fi
+if [[ -n "$TPUT" ]]
+then
+	if [[ -z $PASTEBIN ]]; then
+		FINAL_URL=`tput setaf 1; grep "SUCCESS:" $TEMPDIR/wget.tmp | cut -d ' ' -f 2 ; tput sgr0`
 	else
-		if [[ -z $PASTEBIN ]]; then
-			FINAL_URL=`grep "SUCCESS:" /tmp/alsainfo/wget.tmp | cut -d ' ' -f 2`
-		else
-			FINAL_URL=`grep "SUCCESS:" /tmp/alsainfo/wget.tmp |sed -n 's/.*\:\([0-9]\+\).*/http:\/\/pastebin.ca\/\1/p'`
-		fi
+		FINAL_URL=`tput setaf 1; grep "SUCCESS:" $TEMPDIR/wget.tmp |sed -n 's/.*\:\([0-9]\+\).*/http:\/\/pastebin.ca\/\1/p';tput sgr0`
+	fi
+else
+	if [[ -z $PASTEBIN ]]; then
+		FINAL_URL=`grep "SUCCESS:" $TEMPDIR/wget.tmp | cut -d ' ' -f 2`
+	else
+		FINAL_URL=`grep "SUCCESS:" $TEMPDIR/wget.tmp |sed -n 's/.*\:\([0-9]\+\).*/http:\/\/pastebin.ca\/\1/p'`
 	fi
 fi
+
 #Output the URL of the uploaded file.	
-if [[ -z $DIALOG ]]
-then
-	echo -e "\b Done!"
-	echo ""
-	if [[ -z $NOUPLOAD ]]; then
-		echo "Your ALSA information is located at $FINAL_URL"
-		echo "Please inform the person helping you."
-		echo ""
-	fi
-fi
-if [[ -n $DIALOG ]]
-then
-	if [[ -n $NOUPLOAD ]]; then
-		if [[ -n $PBERROR ]]; then
-			dialog --backtitle "$BGTITLE" --title "Information collected" --msgbox "An error occured while contacting the $WWWSERVICE. Your information was NOT automatically uploaded.\n\nYour ALSA information can be seen by looking in $FILE" 10 100
-		else
-			dialog --backtitle "$BGTITLE" --title "Information collected" --msgbox "You requested that your information was NOT automatically uploaded to the $WWWSERVICE\n\nYour ALSA information can be seen by looking in $FILE" 10 100
-		fi
-	else
-		dialog --backtitle "$BGTITLE" --title "Information uploaded" --yesno "Would you like to see the uploaded information?" 5 100 
-		DIALOG_EXIT_CODE=$?
-	if [ $DIALOG_EXIT_CODE = 0 ]; then
-		grep -v "alsa-info.txt" /tmp/alsa-info.txt >/tmp/alsainfo/uploaded.txt
-		dialog --backtitle "$BGTITLE" --textbox /tmp/alsainfo/uploaded.txt 0 0
-	fi
-	fi
-fi 
-clear
-if [[ -n $NOUPLOAD ]]; then
-	if [[ -n $PBERROR ]]; then
-		echo "An error occured while contacting the $WWWSERVICE. Your information was NOT automatically uploaded."
-		echo ""
-		echo "Your ALSA information can be seen by looking in $FILE"
-		echo ""
-	else
-		echo "You requested that your information was NOT automatically uploaded to the $WWWSERVICE"
-		echo ""
-		echo "Your ALSA information can be seen by looking in $FILE"
-		echo ""
-	fi
-fi
-if [[ -z $NOUPLOAD ]]; then
 echo "Your ALSA information is located at $FINAL_URL"
 echo "Please inform the person helping you."
 echo ""
-fi
-	#We posted the file , so we dont need it anymore. delete it.
-	if [ -z $KEEP_FILES ]
-	then
-		rm $FILE 
-	fi
 
 #We couldnt find a suitable wget, so tell the user to upload manually.
 else
+	mv $FILE $NFILE || exit 1
 	if [[ -z $DIALOG ]]
 	then
 		if [[ -z $PASTEBIN ]]; then
@@ -740,7 +846,7 @@ else
 		echo "    1. Couldnt find 'wget' in your PATH"
 		echo "    2. Your version of wget is less than 1.8.2"
 		echo ""
-		echo "Please manually upload $FILE to http://www.alsa-project.org/cardinfo-db/ and submit your post."
+		echo "Please manually upload $NFILE to http://www.alsa-project.org/cardinfo-db/ and submit your post."
 		echo ""
 		else
 		echo ""
@@ -749,23 +855,16 @@ else
 		echo "    1. Couldnt find 'wget' in your PATH"
 		echo "    2. Your version of wget is less than 1.8.2"
 		echo ""
-		echo "Please manually upload $FILE to http://www.pastebin.ca/upload.php and submit your post."
+		echo "Please manually upload $NFILE to http://www.pastebin.ca/upload.php and submit your post."
 		echo ""
 		fi
-	fi
-	if [[ -n $DIALOG ]]
-	then
+	else
 		if [[ -z $PASTEBIN ]]; then
-			dialog --backtitle "$BGTITLE" --msgbox "Could not automatically upload output to http://www.alsa-project.org.\nPossible reasons are:\n\n    1. Couldn't find 'wget' in your PATH\n    2. Your version of wget is less than 1.8.2\n\nPlease manually upload $FILE to http://www.alsa-project,org/cardinfo-db/ and submit your post." 25 100
+			dialog --backtitle "$BGTITLE" --msgbox "Could not automatically upload output to http://www.alsa-project.org.\nPossible reasons are:\n\n    1. Couldn't find 'wget' in your PATH\n    2. Your version of wget is less than 1.8.2\n\nPlease manually upload $NFILE to http://www.alsa-project,org/cardinfo-db/ and submit your post." 25 100
 		else
-			dialog --backtitle "$BGTITLE" --msgbox "Could not automatically upload output to http://www.pastebin.ca.\nPossible reasons are:\n\n    1. Couldn't find 'wget' in your PATH\n    2. Your version of wget is less than 1.8.2\n\nPlease manually upload $FILE to http://www.pastebin.ca/upload.php and submit your post." 25 100
+			dialog --backtitle "$BGTITLE" --msgbox "Could not automatically upload output to http://www.pastebin.ca.\nPossible reasons are:\n\n    1. Couldn't find 'wget' in your PATH\n    2. Your version of wget is less than 1.8.2\n\nPlease manually upload $NFILE to http://www.pastebin.ca/upload.php and submit your post." 25 100
 		fi
 	fi
 fi
-#Clean up the temp files
-if [ -z $KEEP_FILES ]
-then
-	cleanup
-fi
 
-fi # proceed
+
